'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

import sys
import os
import random
import pygame
import pygame.image

import maze
from maze import Maze
import ega
import data
from giant import Giant
from player import RailsThing
from surfutil import set_colour
from mob import Mob
from fog import Fog

DISPLAY_SIZE = (480,320)
BG_COLOUR = 0x1c1c1c
SCORE_POS = (380, 25)





GOOD_MOBS = 4
BAD_MOBS = 4





#FOLLOW_DELAY = 8

class Key(object):

    def __init__(self, gamestate):
        '''
        :type for_maze: Maze
        '''

        self.gamestate = gamestate
        self.for_maze = gamestate.maze
        p = data.filepath("key.png")
        i = pygame.image.load(p)
        i = i.convert_alpha()
        self.surface = i

        while True:
            node = random.choice(gamestate.maze._data)
            node_x, node_y = node.x, node.y
            if node_x > gamestate.maze.width/2 or node_y > gamestate.maze.height/2:
                break

        self.x = node_x*16
        self.y = node_y*16

        self.follow_sprite = None

    @property
    def hitbox(self):
        return pygame.Rect(self.x+4, self.y+4, 8, 8)
    
    def follow(self, other_sprite):
        self.follow_sprite = other_sprite
        self.follow_index = other_sprite.register_follower(self)

    def update(self):
        if self.follow_sprite is None:
            return
        self.x,self.y = self.follow_sprite.get_follower_pos(self)

    def render(self, to_surface):
        to_surface.blit(self.surface, (self.x,self.y))



class LockedDoor(object):

    def __init__(self, gamestate):
        '''
        :type for_maze: Maze
        '''

        self.gamestate = gamestate
        self.for_maze = gamestate.maze
        p = data.filepath("lock.png")
        i = pygame.image.load(p)
        i = i.convert_alpha()
        self.surface = i

        node_x = gamestate.maze.width-1
        node_y = gamestate.maze.height-1
        self.x = node_x*16
        self.y = node_y*16


    def update(self):
        pass

    def render(self, to_surface):
        to_surface.blit(self.surface, (self.x,self.y))


class GameState(object):

    def __init__(self):
        self.maze = None
        self.mobs = []
        self.player = None
        self.score = 0
        self.lives = 3

    @property
    def all(self):
        if self.player:
            yield self.player
        for x in self.mobs:
            yield x



def title(display_surface):
    display_surface.fill(ega.BLACK)
    pygame.display.flip()


    titlefont = pygame.font.SysFont("Futura", 45)
    titlesurf = titlefont.render("Godspeed You! Mutated Maze", True, ega.BRIGHT_MAGENTA)

    instrfont = pygame.font.SysFont("Futura", 20)
    instrsurf = instrfont.render("Press <SPACE> to start.", True, ega.MAGENTA)

    clock = pygame.time.Clock()
    while True:

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return 'quit'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                return 'quit'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_SPACE:
                return 'playgame'

        display_surface.fill(ega.BLACK)

        title_x = (display_surface.get_width() - titlesurf.get_width())/2
        display_surface.blit(titlesurf, (title_x,50))

        title_x = (display_surface.get_width() - instrsurf.get_width())/2
        display_surface.blit(instrsurf, (title_x,270))

        pygame.display.flip()
        clock.tick(30)



def youwin(display_surface):
    display_surface.fill(ega.BLACK)
    pygame.display.flip()


    titlefont = pygame.font.SysFont("Futura", 100)
    titlesurf = titlefont.render("You WIN!", True, ega.BRIGHT_MAGENTA)

    print 'you are a super player'

    instrfont = pygame.font.SysFont("Futura", 20)
    instrsurf = instrfont.render("Press <SPACE> to continue.", True, ega.MAGENTA)

    clock = pygame.time.Clock()
    while True:

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return 'quit'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                return 'title'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_SPACE:
                return 'title'

        display_surface.fill(ega.BLACK)

        title_x = (display_surface.get_width() - titlesurf.get_width())/2
        display_surface.blit(titlesurf, (title_x,50))

        title_x = (display_surface.get_width() - instrsurf.get_width())/2
        display_surface.blit(instrsurf, (title_x,270))

        pygame.display.flip()
        clock.tick(30)



def playgame(screen):

    background = pygame.Surface(screen.get_size())
    background = background.convert()
    background.fill(BG_COLOUR)


    #font = pygame.font.Font(None, 36)
    #text = font.render("Pummel The Chimp, And Win $$$", 1, (10, 10, 10))
    #textpos = text.get_rect(centerx=background.get_width()/2)
    #background.blit(text, textpos)

    instrfont = pygame.font.SysFont("Futura", 20)

    gamestate = GameState()


    m = maze.Maze(22, 18)
    m.generate()
    maze_surface = pygame.Surface((m.width*16, m.height*16), pygame.constants.SRCALPHA, 32).convert_alpha()

    gamestate.maze = m

    mobs = []
    for x in xrange(10):
        mob = Mob(gamestate)
        if random.choice([True, False, False]):
            mob.toggle_mob_type()
        gamestate.mobs.append(mob)


    keyspr = Key(gamestate)
    raild = RailsThing(gamestate)
    gamestate.player = raild
    f = Fog(gamestate)
    lockspr = LockedDoor(gamestate)

    mutator_surface = pygame.Surface((64, 64), pygame.constants.SRCALPHA, 32).convert_alpha()
    mutator_surface.fill((255, 255, 255, 30))

    # blank screen
    screen.blit(background, (0, 0))
    pygame.display.flip()


    clock = pygame.time.Clock()

    MAZE_POS = (16, 16)

    raw_click_pos = (0,0)
    while True:

        # USER INPUT
        # ======================================

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return 'quit'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                return 'title'
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_g:
                print 'No soup for you GruikInc!'
            #    m.generate()
            #    drawmaze(maze_surface, m)
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                raw_click_pos = pygame.mouse.get_pos()
                print raw_click_pos
                click_pos = (raw_click_pos[0]-MAZE_POS[0], raw_click_pos[1]-MAZE_POS[1])

                r = pygame.Rect(click_pos, mutator_surface.get_size())
                selected = set(m.collides_nodes(r))
                m.regenerate_selected(selected)

        # UPDATE
        # ======================================

        raild.update()
        keyspr.update()
        lockspr.update()

        player_hitbox = raild.hitbox
        for mob in gamestate.mobs:
            mob.update()
            if player_hitbox.colliderect(mob.hitbox):
                mob.follow(raild)

        if player_hitbox.colliderect(keyspr.hitbox):
            keyspr.follow(raild)

        if keyspr.follow_sprite == raild and lockspr.x == raild.x and lockspr.y == raild.y:
            return 'win'

        f.update()
        if f.passed:
            f = Fog(gamestate)

        # RENDER
        # ======================================

        screen.blit(background, (0, 0))

        m.render_to_surface(maze_surface)
        lockspr.render(maze_surface)

        for mob in gamestate.mobs:
            mob.render(maze_surface)
        keyspr.render(maze_surface)

        # make sure we can always see your dude
        raild.render(maze_surface)

        f.render(maze_surface)
        screen.blit(maze_surface, MAZE_POS)

        screen.blit(mutator_surface, pygame.mouse.get_pos())

        scoresurf = instrfont.render("Score: %d"%gamestate.score, True, ega.MAGENTA)
        screen.blit(scoresurf, SCORE_POS)

        pygame.display.flip()

        # TIMING
        # ======================================

        clock.tick(30)




def main():

    #print "Hello from your game's main()"
    #print data.load('sample.txt').read()
    pygame.init()
    screen = pygame.display.set_mode(DISPLAY_SIZE)
    pygame.display.set_caption('Godspeed You! Mutated Maze')
    pygame.mouse.set_visible(0)

    state = 'title'

    while state != 'quit' and state is not None:
        if state is 'title':
            state = title(screen)
        elif state is 'playgame':
            state = playgame(screen)
        elif state is 'win':
            state = youwin(screen)
        else:
            raise Exception("unknown state: "+state)





if __name__ == "__main__":
    main()
