import sys
import os
import random
import pygame
import pygame.image

import ega
import data
from surfutil import set_colour

# mob can be good or bad
class Mob(object):


    def __init__(self, gamestate):
        '''
        :type for_maze: Maze
        '''

        self.gamestate = gamestate
        self.for_maze = gamestate.maze

        self.mob_type = 'friendly'


        p = data.filepath("other_dude.png")
        i = pygame.image.load(p)
        set_colour(i)
        i = i.convert_alpha()
        self.good_surface = i

        p = data.filepath("ghost.png")
        i = pygame.image.load(p)
        set_colour(i, ega.BAD_COLOURS)
        i = i.convert_alpha()
        self.bad_surface = i

        p = data.filepath("exclamation.png")
        i = pygame.image.load(p)
        i = i.convert_alpha()
        self.exclamation_surface = i

        self.surface = self.good_surface

        node = random.choice(gamestate.maze._data)
        self.x,self.y = node.x_px, node.y_px
        self.vx = 0
        self.vy = 0

        self.wanted_direction=None
        self.direction=None


        self.state = 'random'
        self.as_enemy_state = None
        self.exclamation_count = None
        self.chase_vx = 0
        self.chase_vy = 0



    @property
    def hitbox(self):
        return pygame.Rect(self.x+4, self.y+3, 8, 10)


    def toggle_mob_type(self):
        if self.mob_type is 'friendly':
            self.mob_type = 'enemy'
            self.surface = self.bad_surface
            self.state = 'enemy'
            self.enemy_state = None
        elif self.mob_type is 'enemy':
            self.mob_type = 'friendly'
            self.surface = self.good_surface
            self.state = 'random'


    def do_fog_mutate(self, pos):
        if self.state is 'follow_player':
            self.follow_sprite.unfollow(self)

        self.state = 'random'
        self.x,self.y = pos
        self.toggle_mob_type()


    def update_random(self):
        speed = 1

        # otherwise only change if right in middle of node
        if self.x % 16 == 0 and self.y % 16 == 0:
            n = self.for_maze.get_node(self.x//16, self.y//16)
            avail_dir = list(n.avail_directions)
            self.wanted_direction = random.choice(avail_dir)

            if  (self.wanted_direction is 'up' and n.up_open) or \
                (self.wanted_direction is 'down' and n.down_open) or \
                (self.wanted_direction is 'left' and n.left_open) or \
                (self.wanted_direction is 'right' and n.right_open):
                self.direction = self.wanted_direction

        dx,dy=0,0
        if self.direction is 'up':
            dy = -1
            assert self.x%16 == 0
            if self.y%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.up_open:
                    dy=0
        elif self.direction is 'down':
            dy = 1
            assert self.x%16 == 0
            if self.y%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.down_open:
                    dy=0
        elif self.direction is 'left':
            dx = -1
            assert self.y%16 == 0
            if self.x%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.left_open:
                    dx=0
        elif self.direction is 'right':
            dx = 1
            assert self.y%16 == 0
            if self.x%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.right_open:
                    dx=0

        self.x+= dx * speed
        self.y+= dy * speed

        self.follow_sprite = None
        self.follow_pos = []


    def follow(self, other_sprite):
        self.follow_sprite = other_sprite
        self.follow_index = other_sprite.register_follower(self)
        self.state = 'follow_player'


    def update_follow_player(self):
        if self.follow_sprite is None:
            return
        self.x,self.y = self.follow_sprite.get_follower_pos(self)



    def find_player_on(self, getnextnode):

        player_hitbox = self.gamestate.player.hitbox

        current_node = self.for_maze.get_node(self.x/16, self.y/16)
        while current_node is not None:
            if current_node.hitbox.colliderect(player_hitbox):
                return True
            current_node = getnextnode(current_node)

        return False

    def update_as_enemy(self):

        EXCLAMATION_COUNT_INIT = 16

        # if on a row
        if self.y % 16 == 0:
            # do we see the player to the left or right?
            # if yes, set state to chasee
            if self.find_player_on(lambda x: x.left_node if x.left_open else None):
                print 'to the left!'
                self.enemy_state = 'exclamation'
                self.exclamation_count = EXCLAMATION_COUNT_INIT
                self.x >>= 1 # ensure multiple of 2
                self.x <<= 1
                self.chase_vx = -2
                self.chase_vy = 0
            if self.find_player_on(lambda x: x.right_node if x.right_open else None):
                print 'to the right!'
                self.enemy_state = 'exclamation'
                self.exclamation_count = EXCLAMATION_COUNT_INIT
                self.x >>= 1 # ensure multiple of 2
                self.x <<= 1
                self.chase_vx = 2
                self.chase_vy = 0


        # if on a column
        if self.x % 16 == 0:
            # do we see player above or below?
            if self.find_player_on(lambda x: x.up_node if x.up_open else None):
                print 'above you!'
                self.enemy_state = 'exclamation'
                self.exclamation_count = EXCLAMATION_COUNT_INIT
                self.y >>= 1 # ensure multiple of 2
                self.y <<= 1
                self.chase_vx = 0
                self.chase_vy = -2
            if self.find_player_on(lambda x: x.down_node if x.down_open else None):
                print 'below you!'
                self.enemy_state = 'exclamation'
                self.exclamation_count = EXCLAMATION_COUNT_INIT
                self.y >>= 1 # ensure multiple of 2
                self.y <<= 1
                self.chase_vx = 0
                self.chase_vy = 2


        #if see player
        # show exclamation
        # beep!
        # pause for 1/10 second
        # run in that direction until hit a wall..
        # back to random

        if self.enemy_state is 'chase':
            # if at intersection
            if self.x%16==0 and self.y%16 ==0:
                node = self.for_maze.get_node(self.x/16, self.y/16)
                if self.chase_vx < 0:
                    if not node.left_open:
                        self.enemy_state = 'random'
                if self.chase_vx > 0:
                      if not node.right_open:
                        self.enemy_state = 'random'
                if self.chase_vy < 0:
                    if not node.up_open:
                        self.enemy_state = 'random'
                if self.chase_vy > 0:
                      if not node.down_open:
                        self.enemy_state = 'random'

        if self.enemy_state is None or self.enemy_state is 'random':
            self.update_random()
        elif self.enemy_state is 'chase':
            self.x += self.chase_vx
            self.y += self.chase_vy
        elif self.enemy_state is 'exclamation':
            print 'count down'
            self.exclamation_count -= 1
            if self.exclamation_count <= 0:
                print 'chase'
                self.enemy_state = 'chase'



    def update(self):

        if self.state is 'random':
            self.update_random()
        if self.state is 'enemy':
            self.update_as_enemy()
        elif self.state is 'follow_player':
            self.update_follow_player()







    def render(self, to_surface):
        to_surface.blit(self.surface, (self.x, self.y))
        if self.state is 'enemy' and self.enemy_state is 'exclamation':
            to_surface.blit(self.exclamation_surface, (self.x, self.y-8))

