__author__ = 'sonneveld'

import sys
import os
import random
import pygame
import pygame.image

import maze
from maze import Maze
import ega
import data
from giant import Giant

from surfutil import set_colour


FOLLOW_DELAY = 6
MAX_FOLLOWS = 16


class RailsThing(object):


    def __init__(self, gamestate):
        '''
        :type for_maze: Maze
        '''

        self.gamestate = gamestate
        self.for_maze = gamestate.maze
        p = data.filepath("other_dude.png")
        i = pygame.image.load(p)
        set_colour(i)
        self.surface = i.convert_alpha()


        self.x = 0
        self.y = 0
        self.vx = 0
        self.vy = 0

        self.wanted_direction=None
        self.direction=None

        # fill up history with the same pos
        self.pos_history = [ (self.x, self.y) ] * FOLLOW_DELAY*MAX_FOLLOWS

        self.followers = []

    @property
    def hitbox(self):
        return pygame.Rect(self.x+4, self.y+3, 8, 10)

    def add_pos_history(self, pos):

        last = self.pos_history[-1]
        if pos == last:
            return
        self.pos_history.append( pos )
        self.pos_history = self.pos_history[-FOLLOW_DELAY*MAX_FOLLOWS:]


    def register_follower(self, f):
        if f in self.followers:
            return
        self.followers.append(f)

    def unfollow(self, f):
        if f not in self.followers:
            return
        self.followers.remove(f)

    def get_follower_index(self, f):
        return self.followers.index(f)

    def get_follower_pos(self, f):
        hist_index = FOLLOW_DELAY*(self.get_follower_index(f)+1)
        pos = self.pos_history[-hist_index]
        return pos
    
    def update(self):

        """
        the aim here is for 'pacman' like logic where you keep on going but
        you can suggest where you'd like to go if there is an opening.
        """

        pressed = pygame.key.get_pressed()
        if pressed[pygame.K_UP]:
            self.wanted_direction="up"
        if pressed[pygame.K_DOWN]:
            self.wanted_direction="down"
        if pressed[pygame.K_LEFT]:
            self.wanted_direction="left"
        if pressed[pygame.K_RIGHT]:
            self.wanted_direction="right"


        # allow change in opposite direction
        if  (self.direction is 'up' and self.wanted_direction is 'down') or \
            (self.direction is 'down' and self.wanted_direction is 'up') or \
            (self.direction is 'left' and self.wanted_direction is 'right') or \
            (self.direction is 'right' and self.wanted_direction is 'left'):
            self.direction = self.wanted_direction

        # otherwise only change if right in middle of node
        if self.x % 16 == 0 and self.y % 16 == 0:
            n = self.for_maze.get_node(self.x//16, self.y//16)
            if  (self.wanted_direction is 'up' and n.up_open) or \
                (self.wanted_direction is 'down' and n.down_open) or \
                (self.wanted_direction is 'left' and n.left_open) or \
                (self.wanted_direction is 'right' and n.right_open):
                self.direction = self.wanted_direction

        dx,dy=0,0
        if self.direction is 'up':
            dy = -1
            assert self.x%16 == 0
            if self.y%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.up_open:
                    dy=0
        elif self.direction is 'down':
            dy = 1
            assert self.x%16 == 0
            if self.y%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.down_open:
                    dy=0
        elif self.direction is 'left':
            dx = -1
            assert self.y%16 == 0
            if self.x%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.left_open:
                    dx=0
        elif self.direction is 'right':
            dx = 1
            assert self.y%16 == 0
            if self.x%16 == 0:
                maze_x = self.x//16
                maze_y = self.y//16
                n = self.for_maze.get_node(maze_x, maze_y)
                if not n.right_open:
                    dx=0

        self.x+= dx * 2
        self.y+= dy * 2

        self.add_pos_history( (self.x, self.y))




    def render(self, to_surface):
        to_surface.blit(self.surface, (self.x, self.y))




